@@ -101,7 +101,8 @@ tiledb_query_set_subarray <- function(query, subarray, type) {
 #'
 #' This function allocates query buffers directly from R vectors in
 #' case the types match: `integer`, `double`, `logical`. For more
-#' general types see \code{tiledb_query_buffer_alloc_ptr}.
+#' general types see \code{tiledb_query_buffer_alloc_ptr} and
+#' \code{tiledb_query_buffer_assign_ptr}
 #' @param query A TileDB Query object
 #' @param attr A character value containing the attribute
 #' @param buffer A vector providing the query buffer
@@ -285,6 +286,7 @@ tiledb_query_submit_async <- function(query) {
 tiledb_query_finalize <- function(query) {
   stopifnot(query_object=is(query, "tiledb_query"))
   libtiledb_query_finalize(query@ptr)
+  .pkgenv[["query_status"]] <- libtiledb_query_status(query@ptr)
   invisible(query)
 }
 
@@ -298,17 +300,56 @@ tiledb_query_status <- function(query) {
   libtiledb_query_status(query@ptr)
 }
 
-#' Get TileDB Query result buffer elements
+#' Get TileDB Query result buffer element size
+#'
+#' The underlying library functions returns a pair of values as a vector
+#' of length two. The first number is the number of element offsets for variable
+#' size attributes (and always zero for fixed-sized attributes and coordinates).
+#' The second is the number of elements in the data buffer. For variable-sized
+#' attributes the first number is the number of cells read (and hence the number
+#' of offsets), the second number is the number of elements in the data buffer.
+#'
+#' As this function was first made available when only a scalar (corresponding to
+#' the second result) was returned, we still return that value.
 #'
 #' @param query A TileDB Query object
 #' @param attr A character value containing the attribute
 #' @return A integer with the number of elements in the results buffer
 #' for the given attribute
+#' @seealso tiledb_query_result_buffer_elements_vec
 #' @export
 tiledb_query_result_buffer_elements <- function(query, attr) {
   stopifnot(query_object=is(query, "tiledb_query"),
             attr_variable=is.character(attr))
-  libtiledb_query_result_buffer_elements(query@ptr, attr)
+  libtiledb_query_result_buffer_elements(query@ptr, attr, 1) # request 2nd el in pair
+}
+
+#' Get TileDB Query result buffer element size pair as vector
+#'
+#' The underlying library functions returns a pair of values as a vector
+#' of length two. The first number is the number of element offsets for variable
+#' size attributes (and always zero for fixed-sized attributes and coordinates).
+#' The second is the number of elements in the data buffer. For variable-sized
+#' attributes the first number is the number of cells read (and hence the number
+#' of offsets), the second number is the number of elements in the data buffer.
+#' In the case of a nullable attribute, a third element is returned with the size of
+#' the validity buffer.
+#'
+#' @param query A TileDB Query object
+#' @param attr A character value containing the attribute
+#' @param nullable A logical variable that is \sQuote{TRUE} to signal that the attribute
+#' is nullable, and \sQuote{FALSE} otherwise
+#' @return A vector with the number of elements in the offsets buffer (and zero
+#' for fixed-size attribute or dimensions), the number elements in the results
+#' buffer for the given attribute, and (if nullable) a third element with the validity
+#' buffer size.
+#' @seealso tiledb_query_result_buffer_elements
+#' @export
+tiledb_query_result_buffer_elements_vec <- function(query, attr, nullable = FALSE) {
+  stopifnot(`query must be a query_object` = is(query, "tiledb_query"),
+            `attr must be a string` = is.character(attr),
+            `nullable must be a logical` = is.logical(nullable))
+  libtiledb_query_result_buffer_elements_vec(query@ptr, attr, nullable)
 }
 
 #' Set a range for a given query
@@ -478,3 +519,14 @@ tiledb_query_set_condition <- function(query, qc) {
     query@ptr <- libtiledb_query_set_condition(query@ptr, qc@ptr)
     invisible(query)
 }
+
+#' Retrieve the cached status of the last finalized query
+#'
+#' This function accesses the status of the last query without
+#' requiring the query object.
+#'
+#' @return The status of the last query
+#' @export
+tiledb_get_query_status <- function() {
+    .pkgenv[["query_status"]]
+}
