@@ -44,6 +44,10 @@
 #' @slot query_condition A Query Condition object
 #' @slot timestamp_start A POSIXct datetime variable for the inclusive interval start
 #' @slot timestamp_end A POSIXct datetime variable for the inclusive interval start
+#' @slot return_as A character value with the desired \code{tiledb_array} conversion,
+#' permitted values are \sQuote{asis} (default, returning a list of columns),
+#' \sQuote{array}, \sQuote{matrix},\sQuote{data.frame}, \sQuote{data.table}
+#' or \sQuote{tibble}; the latter two require the respective packages installed
 #' @slot ptr External pointer to the underlying implementation
 #' @exportClass tiledb_array
 setClass("tiledb_array",
@@ -63,6 +67,7 @@ setClass("tiledb_array",
                       query_condition = "tiledb_query_condition",
                       timestamp_start = "POSIXct",
                       timestamp_end = "POSIXct",
+                      return_as = "character",
                       ptr = "externalptr"))
 
 #' Constructs a tiledb_array object backed by a persisted tiledb array uri
@@ -98,7 +103,12 @@ setClass("tiledb_array",
 #' at which the array is to be openened. No fragments written earlier will be considered.
 #' @param timestamp_end optional A POSIXct Datetime value determining the inclusive time point
 #' until which the array is to be openened. No fragments written earlier later be considered.
-#' @param ctx tiledb_ctx (optional)
+#' @param return_as optional A character value with the desired \code{tiledb_array} conversion,
+#' permitted values are \sQuote{asis} (default, returning a list of columns), \sQuote{array},
+#' \sQuote{matrix},\sQuote{data.frame}, \sQuote{data.table} or \sQuote{tibble}; the latter
+#' two require the respective packages installed. The existing \code{as.*} arguments take precedent
+#' over this.
+#' @param ctx optional tiledb_ctx
 #' @return tiledb_array object
 #' @export
 tiledb_array <- function(uri,
@@ -117,6 +127,7 @@ tiledb_array <- function(uri,
                          query_condition = new("tiledb_query_condition"),
                          timestamp_start = as.POSIXct(double(), origin="1970-01-01"),
                          timestamp_end = as.POSIXct(double(), origin="1970-01-01"),
+                         return_as = get_return_as_preference(),
                          ctx = tiledb_get_context()) {
   query_type = match.arg(query_type)
   if (!is(ctx, "tiledb_ctx"))
@@ -127,6 +138,8 @@ tiledb_array <- function(uri,
     stop("at most one argument of as.data.frame, as.matrix and as.array can be selected", call. = FALSE)
   if (isTRUE(is.sparse) && as.matrix)
     stop("argument as.matrix cannot be selected for sparse arrays", call. = FALSE)
+  if (sum(as.data.frame, as.matrix, as.array) == 1 && return_as != "asis")
+    return_as <- "asis"
 
   if (length(encryption_key) > 0) {
     if (!is.character(encryption_key))
@@ -183,6 +196,7 @@ tiledb_array <- function(uri,
       query_condition = query_condition,
       timestamp_start = timestamp_start,
       timestamp_end = timestamp_end,
+      return_as = return_as,
       ptr = array_xptr)
 }
 
@@ -235,9 +249,9 @@ setMethod("show", signature = "tiledb_array",
      ,"  is.sparse          = ", if (object@is.sparse) "TRUE" else "FALSE", "\n"
      ,"  as.data.frame      = ", if (object@as.data.frame) "TRUE" else "FALSE", "\n"
      ,"  attrs              = ", if (length(object@attrs) == 0) "(none)"
-                               else paste(object@attrs, collapse=","), "\n"
+                            else paste(object@attrs, collapse=","), "\n"
      ,"  selected_ranges    = ", if (length(object@selected_ranges) > 0) sprintf("(%d non-null sets)", sum(sapply(object@selected_ranges, function(x) !is.null(x))))
-                               else "(none)", "\n"
+                            else "(none)", "\n"
      ,"  extended           = ", if (object@extended) "TRUE" else "FALSE" ,"\n"
      ,"  query_layout       = ", if (length(object@query_layout) == 0) "(none)" else object@query_layout, "\n"
      ,"  datetimes_as_int64 = ", if (object@datetimes_as_int64) "TRUE" else "FALSE", "\n"
@@ -248,6 +262,7 @@ setMethod("show", signature = "tiledb_array",
      ,"  query_condition    = ", if (isTRUE(object@query_condition@init)) "(set)" else "(none)", "\n"
      ,"  timestamp_start    = ", if (length(object@timestamp_start) == 0) "(none)" else format(object@timestamp_start), "\n"
      ,"  timestamp_end      = ", if (length(object@timestamp_end) == 0) "(none)" else format(object@timestamp_end), "\n"
+     ,"  return_as          = '", object@return_as, "'\n"
      ,sep="")
 })
 
@@ -358,10 +373,16 @@ setValidity("tiledb_array", function(object) {
     msg <- c(msg, "The 'ptr' slot does not contain an external pointer.")
   }
 
+  if (!(object@return_as %in% c("asis", "array", "matrix", "data.frame", "data.table", "tibble"))) {
+    valid <- FALSE
+    msg <- c(msg, "The 'return_as' slot must contain one of 'asis', 'array', 'matrix', 'data.frame', 'data.table', 'tibble'.")
+  }
+
   if (valid) TRUE else msg
 
 })
 
+
 ## Internal helper function to map DATETIME_* data to the internal representation (where
 ## we mostly follow NumPy). An example is DATETIME_YEAR where the current year (2021) is
 ## encoded as the offset relative to the _year_ of the epoch, i.e. 51.  When an R user submits
@@ -638,8 +659,17 @@ setMethod("[", "tiledb_array",
       qryptr <- libtiledb_query_set_condition(qryptr, x@query_condition@ptr)
   }
 
-  ## fire off query and close array
+  ## fire off query
   qryptr <- libtiledb_query_submit(qryptr)
+
+  ## check status
+  status <- libtiledb_query_status(qryptr)
+  if (status != "COMPLETE") warning("Query returned '", status, "'.", call. = FALSE)
+
+  ## finalize query
+  qryptr <- libtiledb_query_finalize(qryptr)
+
+  ## close array
   libtiledb_array_close(arrptr)
 
   ## retrieve actual result size (from fixed size element columns)
@@ -659,9 +689,8 @@ setMethod("[", "tiledb_array",
   ## get results
   getResult <- function(buf, name, varnum, resrv, qryptr) {
     if (is.na(varnum)) {
-      sz <- libtiledb_query_result_buffer_elements(qryptr, name, 0)
-      len <- libtiledb_query_result_buffer_elements(qryptr, name, 1)
-      libtiledb_query_get_buffer_var_char(buf, sz, len)[,1]
+      vec <- libtiledb_query_result_buffer_elements_vec(qryptr, name)
+      libtiledb_query_get_buffer_var_char(buf, vec[1], vec[2])[,1]
     } else {
       libtiledb_query_get_buffer_ptr(buf, asint64)
     }
@@ -683,14 +712,29 @@ setMethod("[", "tiledb_array",
       }
   }
 
-  if (!x@as.data.frame && !x@as.matrix && !x@as.array) {
-    res <- as.list(res)
-  } else if (x@as.matrix) {
-    res <- .convertToMatrix(res)
-  } else if (x@as.array) {
-    res <- .convertToArray(dimnames, attrnames, res)
+  if (x@return_as == "asis") {
+      if (!x@as.data.frame && !x@as.matrix && !x@as.array) {
+          res <- as.list(res)
+      } else if (x@as.matrix) {
+          res <- .convertToMatrix(res)
+      } else if (x@as.array) {
+          res <- .convertToArray(dimnames, attrnames, res)
+      }
+  } else if (x@return_as == "array") {       	# if a conversion preference has been given, use it
+      res <- .convertToArray(dimnames, attrnames, res)
+  } else if (x@return_as == "matrix") {
+      res <- .convertToMatrix(res)
+  } else if (x@return_as == "data.frame") {
+      res <- as.data.frame(res)         		# should already be one per above
+  } else if (x@return_as == "data.table" && requireNamespace("data.table", quietly=TRUE)) {
+      res <- data.table::data.table(as.data.frame(res))
+  } else if (x@return_as == "tibble" && requireNamespace("tibble", quietly=TRUE)) {
+      res <- tibble::as_tibble(res)
   }
 
+  ## attach query status
+  attr(res, "query_status") <- .pkgenv[["query_status"]]
+
   invisible(res)
 })
 
@@ -1461,3 +1505,50 @@ setReplaceMethod("return.array",
   validObject(x)
   x
 })
+
+
+
+## -- return_as conversion preference
+
+#' @rdname return_as-tiledb_array-method
+#' @param ... Currently unused
+#' @export
+setGeneric("return_as", function(object, ...) standardGeneric("return_as"))
+
+#' Retrieve return_as conversion preference
+#'
+#' A \code{tiledb_array} object can be returned as a \sQuote{list} (default), \sQuote{array},
+#' \sQuote{matrix}, \sQuote{data.frame}, \sQuote{data.table} or \sQuote{tibble}. This method
+#' permits to select a preference for the returned object. The default value of \sQuote{asis}
+#' means that no conversion is performed.
+#' @param object A \code{tiledb_array} object
+#' @return A character value indicating the preferred conversion where the value is
+#' one of \sQuote{asis} (the default), \sQuote{array}, \sQuote{matrix},\sQuote{data.frame},
+#' \sQuote{data.table}, or \sQuote{tibble}.
+#' @export
+setMethod("return_as",
+          signature = "tiledb_array",
+          function(object) object@return_as)
+
+#' @rdname return_as-set-tiledb_array-method
+#' @export
+setGeneric("return_as<-", function(x, value) standardGeneric("return_as<-"))
+
+#' Retrieve return_as conversion preference
+#'
+#' A \code{tiledb_array} object can be returned as a \sQuote{list} (default), \sQuote{array},
+#' \sQuote{matrix}, \sQuote{data.frame}, \sQuote{data.table} or \sQuote{tibble}. This method
+#' This methods permits to set a preference of returning a \code{list}, \code{array},
+#' \code{matrix}, \code{data.frame}, a \code{data.table}, or a \code{tibble}. The default
+#' value of \dQuote{asis} means that no conversion is performed and a \code{list} is returned.
+#' @param x A \code{tiledb_array} object
+#' @param value A character value with the selection
+#' @return The modified \code{tiledb_array} array object
+#' @export
+setReplaceMethod("return_as",
+                 signature = "tiledb_array",
+                 function(x, value) {
+  x@return_as <- value
+  validObject(x)
+  x
+})
