@@ -68,7 +68,6 @@ struct vfs_fh {
 };
 typedef struct vfs_fh vfs_fh_t;
 
-
 #if TILEDB_VERSION_MAJOR == 2 && TILEDB_VERSION_MINOR < 3
 // we need a placeholder as tiledb::QueryCondition is in at least one function signature
 namespace tiledb {
@@ -77,4 +76,12 @@ namespace tiledb {
 }
 #endif
 
+#if TILEDB_VERSION_MAJOR == 2 && TILEDB_VERSION_MINOR < 2
+// we need a placeholder as tiledb::FragmentInfo is in function signatures
+namespace tiledb {
+    class FragmentInfo {
+    };
+}
+#endif
+
 #endif // __tiledb_h__
